package uoc.tfg.g2tde.controllers;

import java.util.List;
import java.util.Map;
import java.util.Optional;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.security.access.annotation.Secured;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import uoc.tfg.g2tde.entities.Cliente;
import uoc.tfg.g2tde.entities.Consultoria;
import uoc.tfg.g2tde.entities.EmpleadoClienteTarea;
import uoc.tfg.g2tde.interfaces.IClienteService;
import uoc.tfg.g2tde.interfaces.IConsultoriaService;
import uoc.tfg.g2tde.interfaces.IEmpleadoClienteTareaService;
import uoc.tfg.g2tde.interfaces.ITareaService;
import uoc.tfg.g2tde.util.paginator.PageRender;

@Controller
@RequestMapping(value = "/cliente")
@SessionAttributes("cliente")
@Secured({"ROLE_GESTION"})
public class ClienteController {

	@Autowired
	private IClienteService clienteService;
	
	@Autowired
	private IConsultoriaService consultoriaService;
	
	@Autowired
	private IEmpleadoClienteTareaService empleadoClienteTareaService;
	
	@Autowired
	private ITareaService tareaService;
	
	@ModelAttribute("consultorias")
	public List<Consultoria> listaConsultorias(){
		return consultoriaService.findAll();
	}
	
	@Secured({"ROLE_GESTION"})
	@RequestMapping(value = "/listar", method = RequestMethod.GET)
	public String listar(@RequestParam(name = "page", defaultValue = "0") int page, Model model) {
		model.addAttribute("title", "Listado de clientes");
		Pageable pageRequest = PageRequest.of(page, 10);
		
		Page<Cliente> clientes = clienteService.findAll(pageRequest);
		PageRender<Cliente> pageRender = new PageRender<Cliente>("/cliente/listar", clientes);
		model.addAttribute("title", "Listado de clientes");
		model.addAttribute("clientes", clientes);
		model.addAttribute("page", pageRender);
		
		return "/cliente/listar";
	}
	
	@Secured({"ROLE_GESTION"})
	@GetMapping(value = "/detalle/{id}")
	public String detalle(@PathVariable(value = "id") Long id, Map<String, Object> model, RedirectAttributes flash) {
		Optional<Cliente> cliente = clienteService.findById(id);
		if(!cliente.isPresent()) {
			flash.addFlashAttribute("error", "No existe el cliente");
			return "redirect:/cliente/listar";
		}
		model.put("cliente", cliente.get());
		model.put("title", "Detalle cliente");
		return "/cliente/detalle";
	}
	
	@Secured({"ROLE_GESTION"})
	@RequestMapping(value = "/form")
	public String crear(Map<String, Object> model) {
		Cliente cliente = new Cliente();
		model.put("cliente", cliente);
		model.put("title", "Crear Cliente");
		return "/cliente/form";
	}
	
	@Secured({"ROLE_GESTION"})
	@RequestMapping(value="/form/{id}")
	public String editar(@PathVariable(value="id") Long id, Map<String, Object> model) {
		Optional<Cliente> clienteOpt = clienteService.findById(id);
		if(clienteOpt.isPresent()) {
			model.put("cliente", clienteOpt.get());
			model.put("title", "Editar Cliente");
			return "/cliente/form";
		} else {
			return "redirect:/cliente/listar";
		}
	}
	
	@Secured({"ROLE_GESTION"})
	@RequestMapping(value = "/form", method = RequestMethod.POST)
	public String guardar(@Valid Cliente cliente, BindingResult result, Model model, SessionStatus status) {
		if(result.hasErrors()) {
			model.addAttribute("title", "Formulario de cliente");
			return "/cliente/form";
		}
		clienteService.save(cliente);
		status.setComplete();
		return "redirect:/cliente/listar";
	}
	
	@Secured({"ROLE_GESTION"})
	@RequestMapping(value="/eliminar/{id}")
	public String eliminar(@PathVariable(value="id") Long id) {	
		Optional<Cliente> clienteOpt = clienteService.findById(id);
		if(clienteOpt.isPresent()) {
			List<EmpleadoClienteTarea> empleadoClienteTareaList = empleadoClienteTareaService.findByClienteId(id);
			
			clienteService.deleteById(id);
			empleadoClienteTareaList.stream().forEach(e->tareaService.deleteById(e.getTarea().getId()));
		}
		return "redirect:/cliente/listar";
	}
}
