package uoc.tfg.g2tde.controllers;

import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.security.access.annotation.Secured;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import uoc.tfg.g2tde.entities.Criterio;
import uoc.tfg.g2tde.entities.Operador;
import uoc.tfg.g2tde.interfaces.ICriterioService;
import uoc.tfg.g2tde.interfaces.IOperadorService;
import uoc.tfg.g2tde.interfaces.IUsuarioService;
import uoc.tfg.g2tde.util.paginator.PageRender;

@Controller
@RequestMapping(value = "/criterio")
@SessionAttributes("criterio")
@Secured({"ROLE_OPERADOR"})
public class CriterioController {

	@Autowired
	private ICriterioService criterioService;
	
	@Autowired
	private IOperadorService operadorService;
	
	@Autowired
	private IUsuarioService usuarioService;
	
	@ModelAttribute("operadores")
	public List<Operador> listaOperadores(){
		return operadorService.findByTipo("CUANTITATIVO");
	}
	
	@ModelAttribute("tipos")
	public List<String> listaTipos(){
		List<String> tipos = new ArrayList<String>();
		tipos.add("");
		tipos.add("Cliente");
		tipos.add("Empleado");
		tipos.add("Tarea");
		
		return tipos;
	}
	
	@RequestMapping(value = "/tipo/{tipoId}/subtipos/listar", method = RequestMethod.GET)
	public String listarSubtipos(@PathVariable(value = "tipoId") String tipoId, Model model) {		
		List<String> subtipos = new ArrayList<>();
		
		switch(tipoId) {
			case "Empleado":
				subtipos.add("Fecha de contratación");
				break;
			case "Tarea":
				subtipos.add("Tipo");
				subtipos.add("Estimado");
				subtipos.add("Incurrido");
				subtipos.add("Cliente");
				subtipos.add("Empleado");
				break;
		}
		
		model.addAttribute("subtipos", subtipos);
		
		return "/criterio/listarSubtipos";
	}
	
	@RequestMapping(value = "/listar", method = RequestMethod.GET)
	public String listar(@RequestParam(name = "page", defaultValue = "0") int page, Model model) {
		model.addAttribute("title", "Listado de Criterios");
		
		Pageable pageRequest = PageRequest.of(page, 10);
		
		Page<Criterio> criterios = criterioService.findAll(pageRequest);
		PageRender<Criterio> pageRender = new PageRender<Criterio>("/criterio/listar", criterios);
		model.addAttribute("title", "Listado de criterios");
		model.addAttribute("criterios", criterios);
		model.addAttribute("page", pageRender);
		
		return "/criterio/listar";
	}
	
	@RequestMapping(value = "/listarSimple", method = RequestMethod.GET)
	public String listarSimple(Model model) {		
		List<Criterio> criterios = criterioService.findAll();
		model.addAttribute("criterios", criterios.stream()
			.filter(c->"S".equalsIgnoreCase(c.getActivo()))
			.collect(Collectors.toList()));
		
		return "/criterio/listarSimple";
	}
	
	@RequestMapping(value = "/listarCompuesto", method = RequestMethod.GET)
	public String listarCompuesto(Model model) {
		List<Criterio> criterios = criterioService.findAll();
		model.addAttribute("criterios", criterios.stream()
				.filter(c->"S".equalsIgnoreCase(c.getActivo()))
				.collect(Collectors.toList()));
		
		return "/criterio/listarCompuesto";
	}
	
	@GetMapping(value = "/detalle/{id}")
	public String detalle(@PathVariable(value = "id") Long id, Map<String, Object> model, RedirectAttributes flash) {

		Optional<Criterio> criterio = criterioService.findById(id);
		if(!criterio.isPresent()) {
			flash.addFlashAttribute("error", "No existe el criterio");
			return "redirect:/criterio/listar";
		}

		model.put("criterio", criterio.get());
		model.put("title", "Detalle criterio");
		return "/criterio/detalle";
	}
	
	@RequestMapping(value = "/form")
	public String crear(Map<String, Object> model) {

		Criterio criterio = new Criterio();
		model.put("criterio", criterio);
		model.put("title", "Crear criterio");
		return "/criterio/form";
	}
	
	@RequestMapping(value="/form/{id}")
	public String editar(@PathVariable(value="id") Long id, Map<String, Object> model) {
		
		Optional<Criterio> criterio = criterioService.findById(id);
		
		if(criterio.isPresent()) {
			model.put("criterio", criterio.get());
			model.put("title", "Editar Criterio");
		} else {
			return "redirect:/criterio/listar";
		}
		
		return "/criterio/form";
	}
	
	@RequestMapping(value = "/form", method = RequestMethod.POST)
	public String guardar(@Valid Criterio criterio, BindingResult result, Model model, SessionStatus status) {
		
		if(result.hasErrors()) {
			model.addAttribute("title", "Formulario de Criterio");
			return "/criterio/form";
		}
		
		Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
		criterio.setUsuario(usuarioService.findByUsername(authentication.getName()));
		
		criterio.setEsCalculado("N");
		
		criterioService.save(criterio);
		status.setComplete();
		return "redirect:/criterio/listar";
	}
	
	@RequestMapping(value="/eliminar/{id}")
	public String eliminar(@PathVariable(value="id") Long id) {
		
		Optional<Criterio> criterio = criterioService.findById(id);
		
		if(criterio.isPresent()) {
			criterioService.deleteById(id);
		}
		return "redirect:/criterio/listar";
	}
}
