package uoc.tfg.g2tde.controllers;

import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.Optional;
import java.util.stream.Collectors;

import javax.servlet.http.HttpServletRequest;
import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.security.access.annotation.Secured;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import uoc.tfg.g2tde.entities.Conclusion;
import uoc.tfg.g2tde.entities.Criterio;
import uoc.tfg.g2tde.entities.Operador;
import uoc.tfg.g2tde.entities.Regla;
import uoc.tfg.g2tde.entities.ReglaCriterioOperador;
import uoc.tfg.g2tde.interfaces.IConclusionService;
import uoc.tfg.g2tde.interfaces.ICriterioService;
import uoc.tfg.g2tde.interfaces.IOperadorService;
import uoc.tfg.g2tde.interfaces.IReglaService;
import uoc.tfg.g2tde.interfaces.IUsuarioService;
import uoc.tfg.g2tde.services.ReglaCriterioOperadorService;
import uoc.tfg.g2tde.util.paginator.PageRender;

@Controller
@RequestMapping(value = "/regla")
@SessionAttributes("regla")
@Secured({"ROLE_OPERADOR"})
public class ReglaController {

	@Autowired
	private IReglaService reglaService;
	
	@Autowired
	private IUsuarioService usuarioService;
	
	@Autowired
	private ICriterioService criterioService;
	
	@Autowired
	private IConclusionService conclusionService;
	
	@Autowired
	private IOperadorService operadorService;
	
	@Autowired
	private ReglaCriterioOperadorService reglaCriterioOperadorService;
	
	@ModelAttribute("criterios")
	public List<Criterio> listaCriterios(){
		return criterioService.findAll().stream()
			.filter(c->"S".equalsIgnoreCase(c.getActivo()))
			.collect(Collectors.toList());
	}
	
	@ModelAttribute("conclusiones")
	public List<Conclusion> listaConclusiones(){
		return conclusionService.findAll().stream()
			.filter(c->"S".equalsIgnoreCase(c.getActiva()))
			.collect(Collectors.toList());
	}
	
	@ModelAttribute("operadores")
	public List<Operador> listaOperadores(){
		return operadorService.findByTipo("CUALITATIVO")
			.stream().filter(o-> !"YES".equalsIgnoreCase(o.getNombre())).collect(Collectors.toList());
	}
	
	@RequestMapping(value = "/listar", method = RequestMethod.GET)
	public String listar(@RequestParam(name = "page", defaultValue = "0") int page, Model model) {
		model.addAttribute("title", "Listado de Reglas");
		
		Pageable pageRequest = PageRequest.of(page, 10);
		
		Page<Regla> reglas = reglaService.findAll(pageRequest);
		PageRender<Regla> pageRender = new PageRender<Regla>("/regla/listar", reglas);
		model.addAttribute("title", "Listado de reglas");
		model.addAttribute("reglas", reglas);
		model.addAttribute("page", pageRender);
		
		return "/regla/listar";
	}
	
	@GetMapping(value = "/detalle/{id}")
	public String detalle(@PathVariable(value = "id") Long id, Map<String, Object> model, RedirectAttributes flash) {

		Optional<Regla> regla = reglaService.findById(id);
		if (!regla.isPresent()) {
			flash.addFlashAttribute("error", "No existe la regla");
			return "redirect:/regla/listar";
		}

		model.put("regla", regla.get());
		model.put("title", "Detalle regla");
		
		for(ReglaCriterioOperador reglaCriterioOperador: regla.get().getReglaCriterioOperadores()) {
			if("YES".equalsIgnoreCase(reglaCriterioOperador.getOperador().getNombre())) {
				reglaCriterioOperador.getOperador().setNombre(null);
			}
		}
		
		return "/regla/detalle";
	}
	
	@RequestMapping(value = "/form")
	public String crear(Map<String, Object> model) {

		Regla regla = new Regla();
		model.put("regla", regla);
		model.put("title", "Crear regla");
		
		return "/regla/form";
	}
	
	@RequestMapping(value="/form/{id}")
	public String editar(@PathVariable(value="id") Long id, Map<String, Object> model) {
		
		Optional<Regla> regla = reglaService.findById(id);
		
		if(regla.isPresent()) {
			
			model.put("regla", regla.get());
			model.put("title", "Editar Regla");
			
			for(ReglaCriterioOperador reglaCriterioOperador: regla.get().getReglaCriterioOperadores()) {
				if("YES".equalsIgnoreCase(reglaCriterioOperador.getOperador().getNombre())) {
					reglaCriterioOperador.getOperador().setNombre(null);
				}
			}
			
			return "/regla/form";
		} else {
			return "redirect:/regla/listar";
		}
	}
	
	@RequestMapping(value = "/form", method = RequestMethod.POST)
	public String guardar(@Valid Regla regla, BindingResult result, Model model, SessionStatus status, HttpServletRequest request) {
		if(result.hasErrors()) {
			model.addAttribute("title", "Formulario de Regla");
			return "/regla/form";
		}
		
		Authentication authentication = SecurityContextHolder.getContext().getAuthentication();
		regla.setUsuario(usuarioService.findByUsername(authentication.getName()));
		
		regla.setEsCalculada("N");
		
		if(regla.getReglaCriterioOperadores() != null) {
			regla.getReglaCriterioOperadores().stream().forEach(r-> {
				if(null == r.getOperador().getNombre()) {
					r.getOperador().setNombre("YES");
				}
			});
		}
		
		Regla reglaSaved = reglaService.save(regla);
		
		String operadorCompuesto = request.getParameter("operadorCompuesto") != null ? request.getParameter("operadorCompuesto"): "10";
		String criterioCompuesto1 = request.getParameter("reglaCriterioOperadoresCompuesto1");
		String criterioCompuesto2 = request.getParameter("reglaCriterioOperadoresCompuesto2");
		
		if((criterioCompuesto1 != null) || (criterioCompuesto1 != null && criterioCompuesto2 != null)) {
			Optional<Operador> operadorOpt =  operadorService.findById(Long.parseLong(operadorCompuesto));
			ReglaCriterioOperador reglaCriterioOperador = new ReglaCriterioOperador();
			
			if(regla.getReglaCriterioOperadores() != null && !regla.getReglaCriterioOperadores().isEmpty()) {
				Iterator<ReglaCriterioOperador> reglaCriterioOperadorIterator = regla.getReglaCriterioOperadores().iterator();
				while(reglaCriterioOperadorIterator.hasNext()) {
					reglaCriterioOperadorService.deleteById(reglaCriterioOperadorIterator.next().getId());
				}
			}
			
			if(operadorOpt.isPresent()) {
				switch(operadorOpt.get().getNombre()) {
					case "AND":
					case "OR":
						reglaCriterioOperador.setCriterio(criterioService.findById(Long.parseLong(criterioCompuesto1)).get());
						reglaCriterioOperador.setOperador(operadorOpt.get());
						reglaCriterioOperador.setRegla(reglaSaved);
						reglaCriterioOperadorService.save(reglaCriterioOperador);
						
						reglaCriterioOperador = new ReglaCriterioOperador();
						reglaCriterioOperador.setCriterio(criterioService.findById(Long.parseLong(criterioCompuesto2)).get());
						reglaCriterioOperador.setOperador(operadorOpt.get());
						reglaCriterioOperador.setRegla(reglaSaved);
						reglaCriterioOperadorService.save(reglaCriterioOperador);
						break;
					case "NOT":
						reglaCriterioOperador.setCriterio(criterioService.findById(Long.parseLong(criterioCompuesto1)).get());
						reglaCriterioOperador.setOperador(operadorOpt.get());
						reglaCriterioOperador.setRegla(reglaSaved);
						reglaCriterioOperadorService.save(reglaCriterioOperador);
						break;
					default:
						reglaCriterioOperador.setCriterio(criterioService.findById(Long.parseLong(criterioCompuesto1)).get());
						reglaCriterioOperador.setOperador(operadorOpt.get());
						reglaCriterioOperador.setRegla(reglaSaved);
						reglaCriterioOperadorService.save(reglaCriterioOperador);
						break;
				}
			}
			
			status.setComplete();
		}
		return "redirect:/regla/listar";
	}
	
	@RequestMapping(value="/eliminar/{id}")
	public String eliminar(@PathVariable(value="id") Long id) {
		
		Optional<Regla> reglaOpt = reglaService.findById(id);
		
		if(reglaOpt.isPresent()) {
			reglaService.deleteById(id);
		}
		
		return "redirect:/regla/listar";
	}
}
