package uoc.tfg.g2tde.controllers;

import java.util.List;
import java.util.Map;
import java.util.Optional;

import javax.validation.Valid;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.data.domain.Page;
import org.springframework.data.domain.PageRequest;
import org.springframework.data.domain.Pageable;
import org.springframework.security.access.annotation.Secured;
import org.springframework.security.crypto.bcrypt.BCryptPasswordEncoder;
import org.springframework.stereotype.Controller;
import org.springframework.ui.Model;
import org.springframework.validation.BindingResult;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.ModelAttribute;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RequestMethod;
import org.springframework.web.bind.annotation.RequestParam;
import org.springframework.web.bind.annotation.SessionAttributes;
import org.springframework.web.bind.support.SessionStatus;
import org.springframework.web.servlet.mvc.support.RedirectAttributes;

import uoc.tfg.g2tde.entities.Conclusion;
import uoc.tfg.g2tde.entities.Criterio;
import uoc.tfg.g2tde.entities.Empleado;
import uoc.tfg.g2tde.entities.Informe;
import uoc.tfg.g2tde.entities.Regla;
import uoc.tfg.g2tde.entities.Rol;
import uoc.tfg.g2tde.entities.Usuario;
import uoc.tfg.g2tde.interfaces.IConclusionService;
import uoc.tfg.g2tde.interfaces.ICriterioService;
import uoc.tfg.g2tde.interfaces.IEmpleadoService;
import uoc.tfg.g2tde.interfaces.IInformeService;
import uoc.tfg.g2tde.interfaces.IReglaService;
import uoc.tfg.g2tde.interfaces.IRolService;
import uoc.tfg.g2tde.interfaces.IUsuarioService;
import uoc.tfg.g2tde.util.paginator.PageRender;

@Controller
@RequestMapping(value = "/usuario")
@SessionAttributes("usuario")
@Secured({"ROLE_ADMIN"})
public class UsuarioController {

	@Autowired
	private IUsuarioService usuarioService;
	
	@Autowired
	private IRolService rolService;
	
	@Autowired
	private IEmpleadoService empleadoService;
	
	@Autowired
	private BCryptPasswordEncoder passwordEncoder;
	
	@Autowired
	private IConclusionService conclusionService;
	
	@Autowired
	private ICriterioService criterioService;
	
	@Autowired
	private IReglaService reglaService;
	
	@Autowired
	private IInformeService informeService;
	
	@ModelAttribute("roles")
	public List<Rol> listaRoles(){
		return rolService.findAll();
	}
	
	@ModelAttribute("empleados")
	public List<Empleado> listaEmpleados(){
		return empleadoService.findAll();
	}
	
	@RequestMapping(value = "/listar", method = RequestMethod.GET)
	public String listar(@RequestParam(name = "page", defaultValue = "0") int page, Model model) {
		model.addAttribute("title", "Listado de Usuarios");
		
		Pageable pageRequest = PageRequest.of(page, 10);
		
		Page<Usuario> usuarios = usuarioService.findAll(pageRequest);
		PageRender<Usuario> pageRender = new PageRender<Usuario>("/usuario/listar", usuarios);
		model.addAttribute("title", "Listado de usuarios");
		model.addAttribute("usuarios", usuarios);
		model.addAttribute("page", pageRender);
		
		return "/usuario/listar";
	}
	
	@GetMapping(value = "/detalle/{id}")
	public String detalle(@PathVariable(value = "id") String id, Map<String, Object> model, RedirectAttributes flash) {

		Optional<Usuario> usuario = usuarioService.findById(id);
		if (!usuario.isPresent()) {
			flash.addFlashAttribute("error", "No existe el usuario");
			return "redirect:/usuario/listar";
		}

		model.put("usuario", usuario.get());
		model.put("title", "Detalle usuario");
		return "/usuario/detalle";
	}
	
	@RequestMapping(value = "/form")
	public String crear(Map<String, Object> model) {

		Usuario usuario = new Usuario();
		model.put("usuario", usuario);
		model.put("title", "Crear usuario");
		return "/usuario/form";
	}
	
	@RequestMapping(value="/form/{id}")
	public String editar(@PathVariable(value="id") String id, Map<String, Object> model) {
		
		Optional<Usuario> usuarioOpt = usuarioService.findById(id);;
		
		if(usuarioOpt.isPresent()) {
			model.put("usuario", usuarioOpt.get());
			model.put("title", "Editar Usuario");
			return "/usuario/form";
		} else {
			return "redirect:/usuario/listar";
		}
	}
	
	@RequestMapping(value = "/form", method = RequestMethod.POST)
	public String guardar(@Valid Usuario usuario, BindingResult result, Model model, SessionStatus status) {
		
		if(result.hasErrors()) {
			model.addAttribute("title", "Formulario de Usuario");
			return "/usuario/form";
		}
		
		usuario.setPassword(passwordEncoder.encode(usuario.getPassword()));
		usuarioService.save(usuario);
		status.setComplete();
		return "redirect:/usuario/listar";
	}
	
	@RequestMapping(value="/eliminar/{id}")
	public String eliminar(@PathVariable(value="id") String id) {
		
		Optional<Usuario> usuarioOpt = usuarioService.findById(id);
		
		if(usuarioOpt.isPresent()) {
			
			usuarioOpt.get().setEmpleado(null);
			usuarioService.save(usuarioOpt.get());
			
			List<Regla> reglaList = reglaService.findByUsuarioUsername(id);			
			reglaList.stream().forEach(r->{
				r.setUsuario(null);
				reglaService.save(r);
			});
			
			List<Conclusion> conclusionList = conclusionService.findByUsuarioId(id);			
			conclusionList.stream().forEach(c->{
				c.setUsuario(null);
				conclusionService.save(c);
			});
			
			List<Criterio> criterioList = criterioService.findByUsuarioUsername(id);			
			criterioList.stream().forEach(c->{
				c.setUsuario(null);
				criterioService.save(c);
			});
			
			List<Informe> informesList = informeService.findByUsuarioUsername(id);			
			informesList.stream().forEach(i->{
				i.setUsuario(null);
				informeService.save(i);
			});
			
			usuarioService.deleteById(id);
		}
		return "redirect:/usuario/listar";
	}
	
	@Secured({"ROLE_OPERADOR", "ROLE_GESTION", "ROLE_ADMIN"})
	@GetMapping(value = "/detalle/username/{username}")
	public String detalleUsername(@PathVariable(value = "username") String username, Map<String, Object> model, RedirectAttributes flash) {

		Usuario usuario = usuarioService.findByUsername(username);

		model.put("usuario", usuario);
		model.put("title", "Detalle usuario logueado: " + usuario.getUsername());
		return "/usuario/detalle";
	}
}
